﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Common;

namespace HeuristicLab.MainForm.WindowsForms {
  public partial class BreadcrumbControl : UserControl {
    private LinkedList<IContent> breadcrumbs;
    public IEnumerable<IContent> Breadcrumbs {
      get { return breadcrumbs; }
      set {
        breadcrumbs = new LinkedList<IContent>(value);
        BuildBreadcrumbTrail();
      }
    }

    public ViewHost ViewHost { get; set; }

    public BreadcrumbControl() {
      InitializeComponent();
      breadcrumbs = new LinkedList<IContent>();
    }

    private void BuildBreadcrumbTrail() {
      var curLoc = new Point(3, 0);
      int idx = 0;
      var labels = new List<Label>();
      foreach (var bc in breadcrumbs) {
        Label label;
        if (bc == breadcrumbs.Last.Value) {
          label = new Label {
            Location = curLoc,
            AutoSize = true,
            Font = new Font("Microsoft Sans Serif", 7f),
            Text = bc.ToString()
          };
        } else {
          label = new LinkLabel {
            Location = curLoc,
            AutoSize = true,
            Font = new Font("Microsoft Sans Serif", 7f),
            Text = bc.ToString(),
            Tag = bc
          };
          ((LinkLabel)label).LinkClicked += label_LinkClicked;
        }
        labels.Add(label);
        label.Size = label.GetPreferredSize(Size.Empty);
        curLoc.X += label.Size.Width;
        if (++idx < breadcrumbs.Count) {
          var separator = new Label {
            Location = curLoc,
            AutoSize = true,
            Font = new Font("Microsoft Sans Serif", 7f),
            Text = ">>"
          };
          labels.Add(separator);
          separator.Size = separator.GetPreferredSize(Size.Empty);
          curLoc.X += separator.Size.Width;
        }
      }
      Controls.Clear();
      if (curLoc.X <= Width)
        Controls.AddRange(labels.ToArray());
      else
        AddLabelsRightToLeft(labels);
    }

    private void AddLabelsRightToLeft(List<Label> labels) {
      if (!labels.Any()) return;
      var curLoc = new Point(3, 0);
      var ellipsis = new LinkLabel {
        Location = curLoc,
        AutoSize = true,
        Font = new Font("Microsoft Sans Serif", 7f),
        Text = "..."
      };
      ellipsis.Location = curLoc;
      Controls.Add(ellipsis);
      curLoc.X += ellipsis.Size.Width;
      int x = Width;
      int i = labels.Count - 1;
      while (i >= 0) {
        x -= labels[i].Size.Width;
        if (x < curLoc.X) break;
        x -= labels[--i].Size.Width;
        if (x < curLoc.X) break;
        i--;
      }
      if (i == labels.Count - 1) return;
      i += 1 + i % 2;
      ellipsis.Tag = breadcrumbs.ElementAt(i / 2);
      ellipsis.LinkClicked += label_LinkClicked;
      while (i < labels.Count) {
        var l = labels[i++];
        l.Location = curLoc;
        Controls.Add(l);
        curLoc.X += l.Size.Width;
      }
    }

    private void label_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
      var label = (LinkLabel)sender;
      var content = (IContent)label.Tag;
      while (breadcrumbs.Any() && breadcrumbs.Last.Value != content)
        breadcrumbs.RemoveLast();
      var mainForm = MainFormManager.GetMainForm<MainForm>();
      var outermostViewHost = mainForm.GetOutermostControlOfType<ViewHost>(ViewHost);
      bool showBreadcrumbs = outermostViewHost.ShowBreadcrumbs;
      var oldCrumbs = outermostViewHost.Breadcrumbs;
      outermostViewHost.Content = null;
      var viewType = MainFormManager.GetDefaultViewType(content.GetType());
      outermostViewHost.ViewType = viewType;
      outermostViewHost.Content = content;
      outermostViewHost.UpdateBreadcrumbTrail(oldCrumbs, breadcrumbs);
      outermostViewHost.ShowBreadcrumbs = showBreadcrumbs;
    }

    private void BreadcrumbControl_Resize(object sender, EventArgs e) {
      BuildBreadcrumbTrail();
    }
  }
}